searchData={"items":[{"type":"module","title":"Alarmist","doc":"Alarm handler and more\n\nAlarmist provides an `:alarm_handler` implementation that allows you to check\nwhat alarms are currently active and subscribe to alarm status changes.\n\nIt also provides a DSL for defining alarms based on other alarms. See\n`Alarmist.Alarm`.","ref":"Alarmist.html"},{"type":"function","title":"Alarmist.add_managed_alarm/1","doc":"Add a managed alarm\n\nAfter this call, Alarmist will watch for alarms to be set based on the\nsupplied module and set or clear the specified alarm ID. The module must\n`use Alarmist.Alarm`.\n\nCalling this function a multiple times with the same alarm results in\nthe previous alarm being replaced. Alarm subscribers won't receive\nredundant events if the rules are the same.","ref":"Alarmist.html#add_managed_alarm/1"},{"type":"type","title":"Alarmist.alarm/0","doc":"Alarm information\n\nCalls to `:alarm_handler.set_alarm/1` pass an alarm identifier and\ndescription as a 2-tuple. Alarmist stores the description of the most recent\ncall.\n\n`:alarm_handler.set_alarm/1` doesn't enforce the use of 2-tuples. Alarmist\nnormalizes non-2-tuple alarms so that they have empty descriptions.","ref":"Alarmist.html#t:alarm/0"},{"type":"type","title":"Alarmist.alarm_description/0","doc":"Alarm description\n\nThis is optional supplemental information about the alarm. It could contain\nmore information about why it was set. Don't use it to differentiate between\nalarms. Use the alarm ID for that.","ref":"Alarmist.html#t:alarm_description/0"},{"type":"type","title":"Alarmist.alarm_id/0","doc":"Alarm identifier\n\nAlarm identifiers are the unique identifiers of each alarm that can be\nset or cleared.\n\nWhile SASL alarm identifiers can be anything, Alarmist supplies conventions\nso that it can interpret them. This typespec follows those conventions, but\nyou may come across codes that doesn't. Those cases may be ignored or\nmisinterpreted.","ref":"Alarmist.html#t:alarm_id/0"},{"type":"type","title":"Alarmist.alarm_pattern/0","doc":"Patterns for alarm subscriptions\n\nPatterns can be exact matches or use `:_` to match any value in a position.","ref":"Alarmist.html#t:alarm_pattern/0"},{"type":"type","title":"Alarmist.alarm_state/0","doc":"Alarm state\n\nAlarms are in the `:set` state after a call to `:alarm_handler.set_alarm/1`\nand in the `:clear` state after a call to `:alarm_handler.clear_alarm/1`.\nRedundant calls to `:alarm_handler.set_alarm/1` update the alarm description\nand redundant calls to `:alarm_handler.clear_alarm/1` are ignored.","ref":"Alarmist.html#t:alarm_state/0"},{"type":"type","title":"Alarmist.alarm_type/0","doc":"Alarm type\n\nAlarm types are atoms and for Alarmist-managed alarms, they are\nmodule names.","ref":"Alarmist.html#t:alarm_type/0"},{"type":"function","title":"Alarmist.alarm_type/1","doc":"Extract the alarm type from an alarm ID\n\nExamples:\n```elixir\niex> Alarmist.alarm_type(MyAlarm)\nMyAlarm\niex> Alarmist.alarm_type({NetworkBroken, \"eth0\"})\nNetworkBroken\n```","ref":"Alarmist.html#alarm_type/1"},{"type":"function","title":"Alarmist.clear_alarm_level/1","doc":"Clear knowledge of an alarm's level\n\nIf the alarm gets reported after this call, it will be assigned the default\nalarm level, `:warning`.","ref":"Alarmist.html#clear_alarm_level/1"},{"type":"type","title":"Alarmist.compiled_condition/0","doc":"","ref":"Alarmist.html#t:compiled_condition/0"},{"type":"function","title":"Alarmist.get_alarm_ids/1","doc":"Return a list of all active alarm IDs\n\nOptions:\n* `:level` - filter alarms by severity. Defaults to `:info`.","ref":"Alarmist.html#get_alarm_ids/1"},{"type":"function","title":"Alarmist.get_alarms/1","doc":"Return a list of all active alarms\n\nThis returns `{id, description}` tuples. Note that `Alarmist` normalizes\nalarms that were not set as 2-tuples so this may not match calls to\n`:alarm_handler.set_alarm/1`.\n\nOptions:\n* `:level` - filter alarms by severity. Defaults to `:info`.","ref":"Alarmist.html#get_alarms/1"},{"type":"function","title":"Alarmist.info/1","doc":"Print alarm status in a nice table\n\nOptions:\n* `:level` - filter alarms by severity. Defaults to `:info`.\n* `:sort` - `:level`, `:alarm_id`, or `:duration`. Defaults to `:level`.\n* `:ansi_enabled?` - override the default ANSI setting. Defaults to `true`.","ref":"Alarmist.html#info/1"},{"type":"type","title":"Alarmist.info_options/0","doc":"See `Alarmist.info/1`","ref":"Alarmist.html#t:info_options/0"},{"type":"macro","title":"Alarmist.is_alarm_id/1","doc":"","ref":"Alarmist.html#is_alarm_id/1"},{"type":"function","title":"Alarmist.managed_alarm_ids/0","doc":"Return all managed alarm IDs","ref":"Alarmist.html#managed_alarm_ids/0"},{"type":"function","title":"Alarmist.remove_managed_alarm/1","doc":"Remove a managed alarm","ref":"Alarmist.html#remove_managed_alarm/1"},{"type":"opaque","title":"Alarmist.rule/0","doc":"","ref":"Alarmist.html#t:rule/0"},{"type":"function","title":"Alarmist.set_alarm_level/2","doc":"Set or change the alarm level for an alarm\n\nThe alarm can be either for a managed or unmanaged alarm. Once set, that\nalarm will be reported with the specified level.\n\nWhile this can be used with managed alarms, you should normally pass the\ndesired level as an option to `use Alarmist.Alarm` so that it's handled for\nyou.\n\nIt's also possible to set levels for unmanaged alarms in the application\nconfiguration:\n\n```elixir\nconfig :alarmist, alarm_levels: %{MyUnmanagedAlarm => :critical}\n```\n\nNOTE: Changing the alarm level does not change the status of existing alarms\nsince there's no mechanism to go back in time and change reports. Future\nevents will be reported with the new level.","ref":"Alarmist.html#set_alarm_level/2"},{"type":"function","title":"Alarmist.subscribe/1","doc":"Subscribe to alarm status events\n\nEvents will be delivered to the calling process as:\n\n```elixir\n%Alarmist.Event{\n  id: TheAlarmId,\n  state: :set,\n  description: nil,\n  level: :warning,\n  timestamp: -576460712978320952,\n  previous_state: :unknown,\n  previous_timestamp: -576460751417398083\n}\n```","ref":"Alarmist.html#subscribe/1"},{"type":"function","title":"Alarmist.subscribe_all/0","doc":"Subscribe to alarm status events for all alarms\n\nSee `subscribe/1` for the event format.","ref":"Alarmist.html#subscribe_all/0"},{"type":"function","title":"Alarmist.unsubscribe/1","doc":"Unsubscribe the current process from the specified alarm `:set` and `:clear` events","ref":"Alarmist.html#unsubscribe/1"},{"type":"function","title":"Alarmist.unsubscribe_all/0","doc":"Unsubscribe from alarm status events for all alarms\n\n**NOTE:** This will only remove subscriptions created via `subscribe_all/0`, not\nsubscriptions created for individual alarms via `subscribe/1`.","ref":"Alarmist.html#unsubscribe_all/0"},{"type":"module","title":"Alarmist.Alarm","doc":"DSL for defining managed alarms\n\nThe general form is:\n\n```elixir\ndefmodule MyAlarmModule do\n  use Alarmist.Alarm, level: :warning\n\n  alarm_if do\n    AlarmId1 and AlarmId2\n  end\nend\n```\n\nSee `__using__/1` for options to pass to `use Alarmist.Alarm`.  See\n`Alarmist.Ops` for what operations can be included in `alarm_if` block.","ref":"Alarmist.Alarm.html"},{"type":"macro","title":"Alarmist.Alarm.__using__/1","doc":"Define a managed alarm\n\nThe following options can be passed to `use Alarmist.Alarm`:\n\n* `:level` - the alarm severity. See `t:Logger.level/0`. Defaults to\n  `:warning` and can be overridden by `Alarmist.set_alarm_level/2`.\n* `:parameters` a list of atom keys that refine the scope of the alarm. For\n  example, a networking alarm might specify `[:ifname]` to indicate that the\n  alarm pertains to a specific network interface.\n* `:style` the alarm style when parameters are used. Defaults to\n  `:tagged_tuple` to indicate that alarms are tuples where the first element\n  is the alarm type and the subsequent elements are the parameters.","ref":"Alarmist.Alarm.html#__using__/1"},{"type":"macro","title":"Alarmist.Alarm.alarm_if/1","doc":"Define an alarm condition\n\nSee `Alarmist.Ops` for what operations can be included in `alarm_if` block.","ref":"Alarmist.Alarm.html#alarm_if/1"},{"type":"module","title":"Alarmist.Event","doc":"Struct sent to subscribers on property changes\n\n* `:id` - which alarm\n* `:state` - `:set` or `:clear`\n* `:description` - alarm description or `nil` when the alarm has been cleared\n* `:level` - alarm severity if known to Alarmist. Defaults to `:warning`\n* `:timestamp` - the timestamp (`System.monotonic_time/0`) when the changed happened\n* `:previous_state` - the previous value (`:unknown` if no previous information)\n* `:previous_timestamp` - the timestamp when the property changed to\n  `:previous_state`. Use this to calculate how long the property was the\n  previous state.","ref":"Alarmist.Event.html"},{"type":"type","title":"Alarmist.Event.t/0","doc":"","ref":"Alarmist.Event.html#t:t/0"},{"type":"module","title":"Alarmist.Handler","doc":"Alarm handler","ref":"Alarmist.Handler.html"},{"type":"function","title":"Alarmist.Handler.add_managed_alarm/2","doc":"","ref":"Alarmist.Handler.html#add_managed_alarm/2"},{"type":"function","title":"Alarmist.Handler.clear_alarm_level/1","doc":"","ref":"Alarmist.Handler.html#clear_alarm_level/1"},{"type":"function","title":"Alarmist.Handler.handle_event/2","doc":"Registers a new alarm rule at runtime, registering rules with application config is preferred over this.","ref":"Alarmist.Handler.html#handle_event/2"},{"type":"function","title":"Alarmist.Handler.managed_alarm_ids/0","doc":"","ref":"Alarmist.Handler.html#managed_alarm_ids/0"},{"type":"function","title":"Alarmist.Handler.remove_managed_alarm/1","doc":"","ref":"Alarmist.Handler.html#remove_managed_alarm/1"},{"type":"function","title":"Alarmist.Handler.set_alarm_level/2","doc":"","ref":"Alarmist.Handler.html#set_alarm_level/2"},{"type":"module","title":"Alarmist.Ops","doc":"Alarm operations for use with `alarm_if`","ref":"Alarmist.Ops.html"},{"type":"function","title":"Alarmist.Ops.copy/2","doc":"Replicate an alarm status\n\nThis is useful for aliasing alarm names. For example, if one library sets and\nclears an alarm ID that's in its namespace, but another library wants to\nlisten on changes to an alarm ID in its namespace, a copy rule can glue them\ntogether.\n\nExample:\n\n```elixir\ndefmodule NewAlarm do\n  use Alarmist.Alarm\n\n  alarm_if do\n    OriginalAlarm\n  end\nend\n```","ref":"Alarmist.Ops.html#copy/2"},{"type":"function","title":"Alarmist.Ops.debounce/2","doc":"Set an alarm when the input has been set for a specified duration\n\nThis rule removes transient alarms from triggering remediation unnecessarily.\nThis is useful when remediation is expensive or service impacting and the\ninput alarm is somewhat glitchy.\n\nAlarmist already provides some debouncing since alarms that get set and\ncleared in one alarm processing pass are ignored already. This is unreliable,\nthough, and a debounce rule establishes a duration.\n\nAn example of when debouncing is useful is to delay remediation of higher\nlevel alarms like being disconnected from a backend server. There are many\nreasons that a TCP connection could be interrupted and client code probably\nhas some retry logic in it already to reestablish the connection. In this\ncase, it might be good to delay switching to an offline mode for a little bit\nin the hopes that the problem will naturally go away.\n\n\nExample:\n\n```elixir\ndefmodule NewAlarm do\n  use Alarmist.Alarm\n\n  alarm_if do\n    debounce(Alarm1, 1_000)\n  end\nend\n```","ref":"Alarmist.Ops.html#debounce/2"},{"type":"opaque","title":"Alarmist.Ops.engine/0","doc":"","ref":"Alarmist.Ops.html#t:engine/0"},{"type":"function","title":"Alarmist.Ops.hold/2","doc":"Keep an alarm set for a guaranteed amount of time\n\nThis sets an alarm for at least `timeout` milliseconds after it is set. Each\ntime the alarm is set, the timer is restarted.\n\nHold is useful for types of remediation that are time based. I.e., handling\nan alarm means turning something off for a while since turning that feature\nback on when the alarm gets cleared would likely just result in the alarm\nbeing set again. Managing the timeout period via alarms rather than\nprogrammatically lets you manually clear the alarm if you'd like that feature\nenabled again immediately like if you're debugging.\n\n\nExample:\n\n```elixir\ndefmodule NewAlarm do\n  use Alarmist.Alarm\n\n  alarm_if do\n    hold(Alarm1, 1_000)\n  end\nend\n```","ref":"Alarmist.Ops.html#hold/2"},{"type":"function","title":"Alarmist.Ops.intensity/2","doc":"Sets an alarm when the input alarm has been set and cleared too many times\n\nThis type of rule catches flapping alarms where it's desirable to take some\nkind of remediation when they trigger too many times in a row. Intensity is\nmeasured as `count` set/clears in `duration` milliseconds. This is the same\nas supervision restart intensity thresholds.\n\nAn example of an intensity-based alarm is to handle the case when multiple\nnetwork connections are available, but one that should be good is flakey.\nThis happens if a device has both a cellular and a WiFi connection. Normally\nthe WiFi connection is preferred, but if it keeps going up and down, it may\nbe desirable to raise an alarm. That alarm could disable WiFi for a while.\nCombine this with `hold/2` to manage the duration that WiFi is off.\n\n\nExample:\n\n```elixir\ndefmodule NewAlarm do\n  use Alarmist.Alarm\n\n  alarm_if do\n    intensity(Alarm1, 3, 60_000)\n  end\nend\n```","ref":"Alarmist.Ops.html#intensity/2"},{"type":"function","title":"Alarmist.Ops.logical_and/2","doc":"Set an alarm when all of the input alarms are set\n\nThis is useful when remediation is only useful when a lot of things go wrong.\nFor example, if a device has more than one way of accomplishing a task, there\ncould be a specific remediation when one way stops working. However, if every\nway is broken, the device could trigger a more significant remediation.\n\n\nExample:\n\n```elixir\ndefmodule NewAlarm do\n  use Alarmist.Alarm\n\n  alarm_if do\n    Alarm1 and Alarm2\n  end\nend\n```","ref":"Alarmist.Ops.html#logical_and/2"},{"type":"function","title":"Alarmist.Ops.logical_not/2","doc":"Set an alarm when the input alarm is cleared\n\nThis is useful for \"proof-of-life\" alarms where the presence of an alarm is a\ngood thing.\n\nExample:\n\n```elixir\ndefmodule NewAlarm do\n  use Alarmist.Alarm\n\n  alarm_if do\n    not OriginalAlarm\n  end\nend\n```","ref":"Alarmist.Ops.html#logical_not/2"},{"type":"function","title":"Alarmist.Ops.logical_or/2","doc":"Set an alarm when one or more input alarms get set\n\nThis is useful for triggering a generic remediation. An example of this for\nsetting an alarm that indicates that the device is \"unhealthy\" and needs to\nreboot. There are usually many disastrous alarms that when raised really have\nno great remediation other than reboot. This allows a handler to register for\na combined alarm so that it's decoupled from the alarms that trigger it.\n\nExample:\n\n```elixir\ndefmodule NewAlarm do\n  use Alarmist.Alarm\n\n  alarm_if do\n    Alarm1 or Alarm2\n  end\nend\n```","ref":"Alarmist.Ops.html#logical_or/2"},{"type":"extras","title":"Alarmist","doc":"# Alarmist\n\n[![Hex version](https://img.shields.io/hexpm/v/alarmist.svg \"Hex version\")](https://hex.pm/packages/alarmist)\n[![API docs](https://img.shields.io/hexpm/v/alarmist.svg?label=hexdocs \"API docs\")](https://hexdocs.pm/alarmist/Alarmist.html)\n[![CircleCI](https://dl.circleci.com/status-badge/img/gh/smartrent/alarmist/tree/main.svg?style=svg)](https://dl.circleci.com/status-badge/redirect/gh/smartrent/alarmist/tree/main)\n[![REUSE status](https://api.reuse.software/badge/github.com/smartrent/alarmist)](https://api.reuse.software/info/github.com/smartrent/alarmist)\n\nAlarmist builds on Erlang’s\n[alarm_handler](https://www.erlang.org/doc/man/alarm_handler) by adding support\nfor subscriptions, conditional logic, and other advanced features. It is\ndesigned to be non-intrusive and adheres to existing conventions for naming and\nusing alarms. Only the end user's application needs to depend on Alarmist.","ref":"readme.html"},{"type":"extras","title":"What are alarms - Alarmist","doc":"Alarms are different from events. While events can convey any information, an\nalarm conveys a boolean state. The alarm can either be `set` or `clear`. At it's\ncore, here are the calls:\n\n```elixir\niex> :alarm_handler.set_alarm({SomethingIsWrong, \"Some optional description\"})\n\n# Sometime later when Something is no longer wrong.\niex> :alarm_handler.clear_alarm(SomethingIsWrong)\n```\n\nWhen you're at the IEx prompt, you can see the current alarm state in a few ways, but an easy way is to run `Alarmist.info/1`:\n\n```elixir\niex> Alarmist.info\n                                  Set Alarms\nSEVERITY  ALARM ID          LAST CHANGE               DESCRIPTION\nWarning   SomethingIsWrong  2025-05-26 20:08:48 (2s)  Some optional description\n```\n\nLikewise, code should always be able to know the state of the alarm. If your\ncode started after the event was sent, then it would be missed. Of course,\nyou can work around this, but with alarms there's an expectation that the alarm\nstate is always accessible.\n\nAlarmist builds on this and can build off alarms you have to make new ones that\nsummarize or reflect actual situations of concern.","ref":"readme.html#what-are-alarms"},{"type":"extras","title":"When to use alarms - Alarmist","doc":"Alarms are one tool in the fault management toolbox. They give a name to\npersistent conditions that are involved with non-local remediation to clear.\n\nPersistent in this sense means that the alarm continues to exist until reported\notherwise. It is not transient. For example, a supervised GenServer that\ncrashes is a transient fault since its supervisor is going to restart it. An\nissue like a remote server no longer being reachable is persistent. It may\nbecome reachable in a few seconds or hours or more.\n\nNon-local remediation means that the code that sets the alarm does so to either\nhelp or get help from somewhere else like another library or a person. For\nexample, code that monitors a network connection could set an alarm when the\ninternet is unreachable so that UI code could show the issue to a nearby human.","ref":"readme.html#when-to-use-alarms"},{"type":"extras","title":"Alarm IDs - Alarmist","doc":"Erlang's Alarm Handler allows `AlarmId`s to be any Erlang term. While very\nflexible, structure helps and Alarmist supports two `AlarmId` styles:\n\n* Atoms - `InternetDown` or `:disk_full`\n* Tagged tuples - `{NetworkBroken, \"eth0\"}` or `{FancyAlarm, :something, 1}`\n\nAlarmist refers to the atom in atom-only `AlarmId`s or the first element of the\ntuple as the alarm type. Picking the style to use is simple - does the alarm\nneed parameters? No, then atom; yes, then tagged tuple. In practice, avoiding\nparameters seems to end up being enough simpler that if you're unsure, try that\nfirst.\n\nAs a quick reminder, everything in the `AlarmId` is the important part when it\ncomes to subscribing to and working with alarms. The `AlarmDescription` is\ninformational.\n\nAlarms are public API. Alarmist recommends using Elixir modules for alarms\nwhere the module name is the alarm type. The module is a good place for\ndocumentation and helper functions related to the alarm. This also ensures that\nthe alarm can be documented in Hex docs and the like.","ref":"readme.html#alarm-ids"},{"type":"extras","title":"Managed alarms - Alarmist","doc":"One of the major features of `Alarmist` is the ability to compose alarms via\nboolean logic. This simplifies alarm handling code since it's often the case\nthat you don't want to trigger a remediation immediately or a remediation may\nonly be useful if some combination of alarms are set. Another advantage of\ncreating these \"managed\" alarms is code simplification where the Alarmist DSL\ncan make one-liners out of many real world alarm scenarios.\n\nAs before, networking issues make good examples. Home and business networks\nhave some normal hiccups that don't require remediation. Sometimes just waiting\na bit makes the network start working again. Code that detects a network outage\ncan simply set an alarm stating it is down. `Alarmist` provides primitives for\ncreating a managed alarm that doesn't get set until the network is down longer\nthan a user-specified duration. `Alarmist` can also raise that alarm if the\nnetwork bounces up and down frequently since that's also problematic, but in a\nway that the minimum time criteria wouldn't detect.\n\nTo compose alarms using boolean logic, `Alarmist` provides the `alarm_if`\nmacro. The general form is to create an Elixir module with the name of the\nmanaged alarm and then use `alarm_if` to express the criteria for it being set:\n\n```elixir\ndefmodule MyNewAlarm do\n  use Alarmist.Alarm\n\n  alarm_if do\n    InterestingAlarm1 and InterestingAlarm2\n  end\nend\n```\n\nIn this example, `Alarmist` will set `MyNewAlarm` only when both\n`InterestingAlarm1` and `InterestingAlarm2` are set.","ref":"readme.html#managed-alarms"},{"type":"extras","title":"Alarm options - Alarmist","doc":"Alarmist provides the following options for managed alarms:\n\n* `:level` - the severity of the alarm\n* `:style` - how the alarm message is constructed. `:atom` or `:tagged_tuple`\n* `:parameters` - a list of atom keys that define a `:tagged_tuple` alarm","ref":"readme.html#alarm-options"},{"type":"extras","title":"Alarm severity - Alarmist","doc":"Alarmist supports labeling managed alarms with severity levels matching those\nin `t:Logger.level/0`. Alarms default to the `:warning` level and intermediate\nalarms created internally by Alarmist default to `:debug`.\n\nThe following example shows how to set an alarm's severity.\n\n```elixir\ndefmodule MyNewAlarm do\n  use Alarmist.Alarm, level: :info\n\n  alarm_if do\n    ...\n  end\nend\n```\n\nAlarmist includes the severity in alarm status change events and also lets you\nfilter active alarms with `Alarmist.get_alarms/1` and\n`Alarmist.get_alarm_ids/1`.","ref":"readme.html#alarm-severity"},{"type":"extras","title":"Alarm styles - Alarmist","doc":"The way alarms are represented is called their style. These are either atoms\nlike `MyNewAlarm` or tagged tuples like `{NetworkDown, \"eth0\"}`. Alarmist needs\nto know how managed alarms are represented especially in the tagged tuple case\nso that it handles alarm parameters correctly. Alarms following the `:atom`\nstyle don't need any special handling since those are the default.\n\nAn example of a `:tagged_tuple` alarm is the following:\n\n```elixir\ndefmodule NetworkDownAlarm do\n  use Alarmist.Alarm, style: :tagged_tuple, parameters: [:ifname]\n\n  ...\nend\n```\n\nThe use of the `:style` and `:parameters` options is used by Alarmist to\nrepresent this alarm as `{NetworkDownAlarm, ifname}` where `ifname` gets\nreplaced with the network interface name of interest. Of course, Alarmist\ndoesn't know what network interfaces are available, so application code needs\nto call `Alarmist.add_managed_alarm/1` with each possibility. I.e.,\n`Alarmist.add_managed_alarm({NetworkDownAlarm, \"eth0\"})`","ref":"readme.html#alarm-styles"},{"type":"extras","title":"Managed alarm operators - Alarmist","doc":"Managed alarms defined with `alarm_if` support boolean operators and a few\nspecial purpose operators. The following sections document each of these.","ref":"readme.html#managed-alarm-operators"},{"type":"extras","title":"Identity - Alarmist","doc":"Specifying an `AlarmId` by itself creates a new alarm whose state mirrors the\noriginal one. In other words, it creates an alias and is useful for decoupling\nthe naming of alarms between projects.\n\n```elixir\ndefmodule IdenticalAlarm do\n  use Alarmist.Alarm\n\n  alarm_if do\n    SomeOtherAlarmName\n  end\nend\n```","ref":"readme.html#identity"},{"type":"extras","title":"Debounce - Alarmist","doc":"The `debounce/2` function specifies a minimum amount of time for another alarm\nto be set before it is set. This can be used to delay remediation if there's a\nchance that the alarm goes away on its own.\n\n```elixir\ndefmodule RealProblemAlarm do\n  use Alarmist.Alarm\n\n  alarm_if do\n    # Set this module's alarm when FlakyAlarm has been set for at for 5 seconds\n    debounce(FlakyAlarm, 5_000)\n  end\nend\n```","ref":"readme.html#debounce"},{"type":"extras","title":"Hold - Alarmist","doc":"The `hold/2` function specifies a minimum amount of time for the new alarm to\nbe set. For example, if an alarm triggers an indicator on a UI, then it may\nneed to stay on for a minimum duration. While the UI could have the timer,\ncreating an alarm lets other code or alarms change their behavior as well.\n\n```elixir\ndefmodule LongerAlarm do\n  use Alarmist.Alarm\n\n  alarm_if do\n    # Set the alarm for at least 3 seconds whenever FlakyAlarm\n    hold(FlakyAlarm, 3_000)\n  end\nend\n```","ref":"readme.html#hold"},{"type":"extras","title":"Intensity - Alarmist","doc":"The `intensity/3` function sets an alarm when another has been set and cleared\ntoo many times in a row. The metric is set/cleared x times in y milliseconds\nsimilar to OTP's supervisor restart intensity parameters. It can be useful to\ncombine `intensity/3` with `hold/2` to create an alarm that disables a feature\nfor a short time when it flaps too much. Some people call this a penalty box.\n\n```elixir\ndefmodule IntensityThresholdAlarm do\n  use Alarmist.Alarm\n\n  alarm_if do\n    # Set when raised and cleared >= 5 times in 3 seconds\n    intensity(FlakyAlarm, 5, 3_000)\n  end\nend\n```","ref":"readme.html#intensity"},{"type":"extras","title":"Boolean logic - Alarmist","doc":"Standard Elixir boolean operators like `and`, `or`, and `not` can be used to\ncombine and group multiple alarms. This is an easy way to create an alarm that\ntracks exactly what you want.\n\n```elixir\ndefmodule IntensityThresholdAlarm do\n  use Alarmist.Alarm\n\n  alarm_if do\n    (Alarm1 or Alarm2) and intensity(FlakyAlarm, 5, 10_000)\n  end\nend\n```","ref":"readme.html#boolean-logic"},{"type":"extras","title":"Example - Alarmist","doc":"The following example shows how to define an alarm that WiFi is unstable based\non a alarm that says when WiFi is down. This is a real life example of an\nembedded device with an expensive backup cellular connection. WiFi can be\nflaky, though, so you wouldn't want to turn on the cellular connection right\nwhen WiFi goes down since that might be a hiccup.\n\nThe following code defines a managed alarm for unstable WiFi,\n`Demo.WiFiUnstable`. The timeouts are short to make it easier to copy/paste\ninto an IEx prompt and manually run.\n\n```elixir\ndefmodule Demo.WiFiUnstable do\n  @moduledoc \"\"\"\n  Alarm for when WiFi bounces too frequently\n  \"\"\"\n  use Alarmist.Alarm\n\n  # WiFi must be down for at least 15 seconds or flapped 2 times in 60 seconds\n  alarm_if do\n    debounce(Demo.WiFiDown, :timer.seconds(15)) or\n      intensity(Demo.WiFiDown, 2, :timer.seconds(60))\n  end\nend\n\ndefmodule Demo do\n  @moduledoc \"\"\"\n  Helpers for setting and clearing alarms\n  \"\"\"\n  def wifi_down() do\n    :alarm_handler.set_alarm({Demo.WiFiDown, nil})\n  end\n\n  def wifi_up() do\n    :alarm_handler.clear_alarm(Demo.WiFiDown)\n  end\n\n  def wifi_flap() do\n    wifi_down()\n    wifi_up()\n    wifi_down()\n    wifi_up()\n  end\nend\n```\n\nNow that we have alarm logic and helpers defined the managed alarm needs to be\nregistered:\n\n```elixir\n  # ... normally in an Application.start or other code that runs on init ...\n  Alarmist.add_managed_alarm(Demo.WiFiUnstable)\n```\n\nThen subscribe for notifications:\n\n```elixir\n  # ... normally in the GenServer with the remediation code...\n  Alarmist.subscribe(Demo.WiFiUnstable)\n```\n\nFinally, we can exercise setting and clearing the alarm:\n\n```elixir\niex> Demo.wifi_flap\n:ok\niex> flush\n%Alarmist.Event{\n  id: Demo.WiFiUnstable,\n  state: :set,\n  description: nil,\n  timestamp: -576460712978320952,\n  previous_state: :unknown,\n  previous_timestamp: -576460751417398083\n}\n:ok\n# Wait ~60 seconds\niex> flush\n%Alarmist.Event{\n  id: Demo.WiFiUnstable,\n  state: :clear,\n  timestamp: -576460652977733801,\n  previous_state: :set,\n  previous_timestamp: -576460712978320952\n}\n```","ref":"readme.html#example"},{"type":"extras","title":"Configuration via the application environment - Alarmist","doc":"It's possible to define managed alarms to add when the Alarmist application\nstarts. This has some convenience if you'd prefer to list all managed alarms in\nyour `config.exs` rather than distribute their registration to runtime.\n\n```elixir\nconfig :alarmist,\n  managed_alarms: [FirstManagedAlarm, SecondManagedAlarm],\n  alarm_levels: %{{:disk_almost_full, ~c\"/\"} => :debug}\n```\n\nWhen Alarmist starts, it will force those modules to be loaded. Alarmist skips\nany alarm modules that have issues and just logs an error.","ref":"readme.html#configuration-via-the-application-environment"},{"type":"extras","title":"License - Alarmist","doc":"Alarmist is licensed under the Apache License, Version 2.0.","ref":"readme.html#license"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nThis project follows [Semantic Versioning](https://semver.org/spec/v2.0.0.html).","ref":"changelog.html"},{"type":"extras","title":"v0.3.1 - 2025-06-10 - Changelog","doc":"* Updates\n  * Loosen `:tablet` dependency to allow updates to latest version\n  * Fix Elixir 1.19 warning","ref":"changelog.html#v0-3-1-2025-06-10"},{"type":"extras","title":"v0.3.0 - 2025-05-30 - Changelog","doc":"This is a breaking update that renames terminology and begins feature updates\nbased on experiences over the past year. Here's a summary of the terminology\nchanges:\n\n1. Synthetic alarms are now called managed alarms since they are managed by\n   Alarmist.\n2. `Alarmist.Definition` is now `Alarmist.Alarm` to signify that you're\n   defining alarms.\n3. `defalarm` is now `alarm_if`. The way to read this is \"[Alarmist] sets an\n   alarm IF the following condition is true\". Future releases will have other\n   ways of indicating when alarms should be set.\n\nWhen upgrading, you'll get compiler errors to guide you on the renames.\n\nIn addition to the above breaking changes, there are quite a few updates from\nv0.2.2:\n\n* New features\n  * Support for tuple-based alarm IDs like `{NetworkBroken, \"eth0\"}` to allow\n    for generic alarm types.  The boolean logic for combining alarms supports\n    variables now so that managed alarms don't need to know the\n    instance-specific pieces until registration.\n  * Support alarm severities. Alarm severities use the same atoms as Logger\n    severities (`:error`, `:warning`, `:info`, etc.) and may be set on both\n    managed and unmanaged alarms.\n  * Add `Alarmist.info/1` for quickly getting a list of set and cleared alarms\n    when using the CLI\n  * Support registering alarms and setting levels using the application\n    environment via `config :alarmist, ...`\n\n* Updates\n  * Internally created alarms when registering managed alarms are now all\n    `:debug` severity and won't display or be returned by default since most\n    alarm querying functions return `:info` and higher. You can still get to\n    them by passing `level: :debug` to affected functions.\n  * Fix silent failures of Alarmist API calls immediately after Alarmist is\n    started due to async `gen_event` handler registration. This seemed to only\n    affect unit tests in practice.\n  * Clean up state better when unregistering managed alarms and stopping the\n    Alarmist app.\n  * Many more unit tests for better code coverage of edge cases","ref":"changelog.html#v0-3-0-2025-05-30"},{"type":"extras","title":"v0.2.2 - 2025-04-24 - Changelog","doc":"* Updates\n  * Fixed dropped alarm descriptions that were reported before Alarmist starts\n  * Fixed `Alarmist.remove_synthetic_alarm/1` to actually work. It doesn't look\n    like this function is actually used in practice, but it will work now.\n  * Add `Alarmist.synthetic_alarm_ids/0` to list what's been registered\n  * Add `Alarmist.subscribe_all/0` and `Alarmist.unsubscribe_all/0` for ease of\n    subscribing to all events\n  * Gracefully handle redundant alarm registrations. These can happen on\n    supervision tree restart. Extra notifications aren't sent and if alarm\n    conditions actually did change on the re-registration, the new ones would be\n    used.\n  * Alarm modules now have a `__get_alarm_def__/0` function for getting the\n    alarm condition source code","ref":"changelog.html#v0-2-2-2025-04-24"},{"type":"extras","title":"v0.2.1 - 2025-03-27 - Changelog","doc":"* Updates\n  * Fix serious issue with incorrect clearing of timers that affects synthetic\n    alarms that use timers such as Debounce, Intensity, and Hold. Timeouts could\n    be missed. Thanks to @x4lldux for reporting the issue.\n  * Improve compile-time checks for `defalarm`\n  * Various documentation improvements and an example\n  * Update licensing and copyright for [REUSE compliance](https://reuse.software/)","ref":"changelog.html#v0-2-1-2025-03-27"},{"type":"extras","title":"v0.2.0 - 2024-12-09 - Changelog","doc":"This is a backwards incompatible update. The following changes are needed:\n\n1. Replace all calls to `Alarmist.current_alarms/0` with\n   `Alarmist.get_alarm_ids/0`. This is a hard deprecation.\n2. Update all message handling on Alarmist events to expect and use the\n   `Alarmist.Event.t()`. In most usage, this means matching on `:id` and\n   `:state` and it should simplify the handling functions.\n\n* Updates\n  * Simplify the API by completely abstracting away the internal implementation\n    with uses the PropertyTables library. This will allow for further internal\n    improvements without forcing breaking changes.\n  * Align the API for getting alarms with `:alarm_handler`. This added the\n    `Alarmist.get_alarms/0` function.\n  * Remove a race condition involving the use of alarm descriptions.\n    Descriptions are sent to subscribers with the alarm status change\n    notification now.\n  * Use one timestamp value for all alarms that were set at initialization time.\n    This removes the ambiguity of whether an alarm changed a few milliseconds\n    after Alarmist start up or was one of the original alarms.\n  * Report when alarms are in an `:unknown` state when no information is\n    available. This is useful for the `:previous_state` field in alarm events.","ref":"changelog.html#v0-2-0-2024-12-09"},{"type":"extras","title":"v0.1.3 - 2024-09-26 - Changelog","doc":"* Updates\n  * Don't crash on non-atom Alarm IDs. Alarmist doesn't support these yet so\n    they're currently ignored.","ref":"changelog.html#v0-1-3-2024-09-26"},{"type":"extras","title":"v0.1.2 - 2024-03-04 - Changelog","doc":"First public release","ref":"changelog.html#v0-1-2-2024-03-04"},{"type":"extras","title":"v0.1.1 - 2024-02-29 - Changelog","doc":"* Updates\n  * Delay swapping alarm handler until supervision tree started to fix possible\n    crash on startup","ref":"changelog.html#v0-1-1-2024-02-29"},{"type":"extras","title":"v0.1.0 - 2024-01-19 - Changelog","doc":"Initial release","ref":"changelog.html#v0-1-0-2024-01-19"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.1"}}